@@ -26,7 +26,6 @@ const Gettext = imports.gettext;
 const _ = imports.gettext.gettext;
 
 // Import versions go here
-imports.gi.versions.GtkClutter = '1.0';
 imports.gi.versions.GdPrivate = '1.0';
 imports.gi.versions.Gd = '1.0';
 imports.gi.versions.Tracker = '0.16';
@@ -34,7 +33,6 @@ imports.gi.versions.TrackerMiner = '0.16';
 imports.gi.versions.EvinceDocument = '3.0';
 imports.gi.versions.Goa = '1.0';
 
-const GtkClutter = imports.gi.GtkClutter;
 const EvDoc = imports.gi.EvinceDocument;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
@@ -60,7 +58,6 @@ const Selections = imports.selections;
 const ShellSearchProvider = imports.shellSearchProvider;
 const TrackerController = imports.trackerController;
 const TrackerUtils = imports.trackerUtils;
-const Tweener = imports.util.tweener;
 const Utils = imports.utils;
 const WindowMode = imports.windowMode;
 
@@ -304,9 +301,8 @@ const Application = new Lang.Class({
         this.parent();
         String.prototype.format = Format.format;
 
-        GtkClutter.init(null);
+        Gtk.init(null);
         EvDoc.init();
-        Tweener.init();
 
         let resource = Gio.Resource.load(Path.RESOURCE_DIR + '/gnome-documents.gresource');
         resource._register();
@@ -375,6 +371,12 @@ const Application = new Lang.Class({
               window_mode: WindowMode.WindowMode.PREVIEW },
             { name: 'view-current',
               window_mode: WindowMode.WindowMode.EDIT },
+            { name: 'present-current',
+              window_mode: WindowMode.WindowMode.PREVIEW,
+              callback: this._onActionToggle,
+              state: GLib.Variant.new('b', false),
+              accel: 'F5',
+            },
             { name: 'print-current', accel: '<Primary>p',
               callback: this._onActionPrintCurrent,
               window_mode: WindowMode.WindowMode.PREVIEW },
@@ -448,9 +450,7 @@ const Application = new Lang.Class({
         return 0;
     },
 
-    _onWindowDestroy: function(window) {
-        this._mainWindow = null;
-
+    _clearState: function() {
         // clean up signals
         changeMonitor.disconnectAll();
         documentManager.disconnectAll();
@@ -468,6 +468,14 @@ const Application = new Lang.Class({
         this._stopMiners();
     },
 
+    _onWindowDestroy: function(window) {
+        this._mainWindow = null;
+
+        // clear our state in an idle, so other handlers connected
+        // to 'destroy' have the chance to perform their cleanups first
+        Mainloop.idle_add(Lang.bind(this, this._clearState));
+    },
+
     _onActivateResult: function(provider, urn, terms) {
         this._createWindow();
         modeController.setWindowMode(WindowMode.WindowMode.PREVIEW);
