@@ -17,15 +17,14 @@
  *
  */
 
-const Clutter = imports.gi.Clutter;
 const WebKit = imports.gi.WebKit;
 const Soup = imports.gi.Soup;
+const Gd = imports.gi.Gd;
 const GdPrivate = imports.gi.GdPrivate;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 const _ = imports.gettext.gettext;
 
 const Lang = imports.lang;
@@ -33,7 +32,6 @@ const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 
 const Application = imports.application;
-const Tweener = imports.util.tweener;
 const MainToolbar = imports.mainToolbar;
 const Searchbar = imports.searchbar;
 const Utils = imports.utils;
@@ -46,21 +44,20 @@ const _BLANK_URI = "about:blank";
 const EditView = new Lang.Class({
     Name: 'EditView',
 
-    _init: function(overlayLayout) {
+    _init: function() {
         this._uri = null;
 
         this.widget = new Gtk.Overlay();
 
         this._scrolledWindow = new Gtk.ScrolledWindow({ hexpand: true,
-                                                        vexpand: true,
-                                                        shadow_type: Gtk.ShadowType.IN });
+                                                        vexpand: true });
         this.widget.get_style_context().add_class('documents-scrolledwin');
         this.widget.add(this._scrolledWindow);
 
         this._session = WebKit.get_default_session ();
         Soup.Session.prototype.add_feature.call(this._session, new Soup.ProxyResolverDefault());
         Soup.Session.prototype.remove_feature.call(this._session, new Soup.CookieJar());
-        let jarfile = GLib.build_filenamev([GLib.get_user_cache_dir(), 'gnome-documents', 'cookies.sqlite'])
+        let jarfile = GLib.build_filenamev([GLib.get_user_cache_dir(), 'gnome-documents', 'cookies.sqlite']);
         this._cookieJar = new Soup.CookieJarDB({ filename: jarfile, read_only: false });
         Soup.Session.prototype.add_feature.call(this._session, this._cookieJar);
 
@@ -186,19 +183,20 @@ const EditToolbar = new Lang.Class({
 
         // back button, on the left of the toolbar
         let iconName =
-            (this.widget.get_direction() == Gtk.TextDirection.RTL) ?
+            (this.toolbar.get_direction() == Gtk.TextDirection.RTL) ?
             'go-next-symbolic' : 'go-previous-symbolic';
-        let backButton =
-            this.widget.add_button(iconName, _("Back"), true);
+        let backButton = new Gd.HeaderSimpleButton({ symbolic_icon_name: iconName,
+                                                     label: _("Back") });
+        this.toolbar.pack_start(backButton);
         backButton.connect('clicked', Lang.bind(this,
             function() {
                 Application.documentManager.setActiveItem(null);
             }));
 
-        let viewButton =
-            this.widget.add_button(null, _("View"), false);
+        let viewButton = new Gd.HeaderSimpleButton({ label: _("View"),
+                                                     action_name: 'app.view-current' });
         viewButton.get_style_context().add_class('suggested-action');
-        viewButton.set_action_name('app.view-current');
+        this.toolbar.pack_end(viewButton);
 
         this._setToolbarTitle();
         this.widget.show_all();
@@ -218,6 +216,6 @@ const EditToolbar = new Lang.Class({
         if (doc)
             primary = doc.name;
 
-        this.widget.set_labels(primary, null);
+        this.toolbar.set_title(primary);
     }
 });
