@@ -21,7 +21,6 @@
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
-const Tweener = imports.util.tweener;
 
 const Application = imports.application;
 const MainToolbar = imports.mainToolbar;
@@ -33,13 +32,12 @@ const View = imports.view;
 const WindowMode = imports.windowMode;
 const Documents = imports.documents;
 
-const Clutter = imports.gi.Clutter;
 const EvView = imports.gi.EvinceView;
+const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 const _ = imports.gettext.gettext;
 
 const _ICON_SIZE = 128;
@@ -49,8 +47,6 @@ const SpinnerBox = new Lang.Class({
     Name: 'SpinnerBox',
 
     _init: function() {
-        this._delayedMoveId = 0;
-
         this.widget = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
                                      row_spacing: 24,
                                      hexpand: true,
@@ -58,13 +54,6 @@ const SpinnerBox = new Lang.Class({
                                      halign: Gtk.Align.CENTER,
                                      valign: Gtk.Align.CENTER });
 
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            opacity: 255,
-                                            x_align: Clutter.ActorAlign.FILL,
-                                            x_expand: true,
-                                            y_align: Clutter.ActorAlign.FILL,
-                                            y_expand: true });
-
         this._spinner = new Gtk.Spinner({ width_request: _ICON_SIZE,
                                           height_request: _ICON_SIZE,
                                           halign: Gtk.Align.CENTER,
@@ -76,56 +65,15 @@ const SpinnerBox = new Lang.Class({
                                       halign: Gtk.Align.CENTER,
                                       valign: Gtk.Align.CENTER });
         this.widget.add(this._label);
-
-        this.widget.connect('destroy', Lang.bind(this, this._clearDelayId));
         this.widget.show_all();
     },
 
-    _clearDelayId: function() {
-        if (this._delayedMoveId != 0) {
-            Mainloop.source_remove(this._delayedMoveId);
-            this._delayedMoveId = 0;
-        }
-    },
-
-    moveIn: function() {
-        this._clearDelayId();
-
-        let parent = this.actor.get_parent();
-        parent.set_child_above_sibling(this.actor, null);
-
+    start: function() {
         this._spinner.start();
-
-        Tweener.addTween(this.actor, { opacity: 255,
-                                       time: 0.30,
-                                       transition: 'easeOutQuad' });
     },
 
-    moveOut: function() {
-        this._clearDelayId();
-
-        Tweener.addTween(this.actor, { opacity: 0,
-                                       time: 0.30,
-                                       transition: 'easeOutQuad',
-                                       onComplete: function () {
-                                           let parent = this.actor.get_parent();
-                                           parent.set_child_below_sibling(this.actor, null);
-
-                                           this._spinner.stop();
-                                       },
-                                       onCompleteScope: this });
-    },
-
-    moveInDelayed: function(delay) {
-        this._clearDelayId();
-
-        this._delayedMoveId = Mainloop.timeout_add(delay, Lang.bind(this,
-            function() {
-                this._delayedMoveId = 0;
-
-                this.moveIn();
-                return false;
-            }));
+    stop: function() {
+        this._spinner.stop();
     }
 });
 
@@ -162,13 +110,6 @@ const ErrorBox = new Lang.Class({
         this.widget.add(this._secondaryLabel);
 
         this.widget.show_all();
-
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            opacity: 255,
-                                            x_align: Clutter.ActorAlign.FILL,
-                                            x_expand: true,
-                                            y_align: Clutter.ActorAlign.FILL,
-                                            y_expand: true });
     },
 
     update: function(primary, secondary) {
@@ -177,26 +118,6 @@ const ErrorBox = new Lang.Class({
 
         this._primaryLabel.label = primaryMarkup;
         this._secondaryLabel.label = secondaryMarkup;
-    },
-
-    moveIn: function() {
-        let parent = this.actor.get_parent();
-        parent.set_child_above_sibling(this.actor, null);
-
-        Tweener.addTween(this.actor, { opacity: 255,
-                                       time: 0.30,
-                                       transition: 'easeOutQuad' });
-    },
-
-    moveOut: function() {
-        Tweener.addTween(this.actor, { opacity: 0,
-                                       time: 0.30,
-                                       transition: 'easeOutQuad',
-                                       onComplete: function () {
-                                           let parent = this.actor.get_parent();
-                                           parent.set_child_below_sibling(this.actor, null);
-                                       },
-                                       onCompleteScope: this });
     }
 });
 
@@ -235,8 +156,6 @@ const EmptyResultsBox = new Lang.Class({
             this._addSystemSettingsLabel();
         }
 
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            opacity: 255 });
         this.widget.show_all();
     },
 
@@ -284,34 +203,6 @@ const EmptyResultsBox = new Lang.Class({
     }
 });
 
-const EmbedWidget = new Lang.Class({
-    Name: 'EmbedWidget',
-    Extends: GtkClutter.Embed,
-
-    _init: function() {
-        this.parent({ use_layout_size: true,
-                      can_focus: false });
-    },
-
-    /* We overide all keyboard handling of GtkClutter.Embed, as it interfers
-     * with the key event propagation and thus focus navigation in gtk+.
-     * We also make the embed itself non-focusable, as we want to treat it
-     * like a container of Gtk+ widget rather than an edge widget which gets
-     * keyboard events.
-     * This means we will never get any Clutter key events, but that is
-     * fine, as all our keyboard input is into GtkClutterActors, and clutter
-     * is just used as a nice way of animating and rendering Gtk+ widgets
-     * and some non-active graphical things.
-     */
-    vfunc_key_press_event: function(event) {
-        return false;
-    },
-
-    vfunc_key_release_event: function(event) {
-        return false;
-    }
-});
-
 const Embed = new Lang.Class({
     Name: 'Embed',
 
@@ -319,68 +210,41 @@ const Embed = new Lang.Class({
         this._queryErrorId = 0;
         this._noResultsChangeId = 0;
 
-        this.widget = new EmbedWidget();
-        this.widget.show();
+        this.widget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
+                                    visible: true });
 
-        // the embed is a vertical ClutterBox
-        let stage = this.widget.get_stage();
-        this._overlayLayout = new Clutter.BinLayout();
-        this.actor = new Clutter.Box({ layout_manager: this._overlayLayout });
-        this.actor.add_constraint(
-            new Clutter.BindConstraint({ coordinate: Clutter.BindCoordinate.SIZE,
-                                         source: stage }));
-        stage.add_actor(this.actor);
+        this._stackOverlay = new Gtk.Overlay({ visible: true });
+        this.widget.pack_end(this._stackOverlay, true, true, 0);
 
-        this._contentsLayout = new Clutter.BoxLayout({ vertical: true });
-        this._contentsActor = new Clutter.Box({ layout_manager: this._contentsLayout });
-        this._overlayLayout.add(this._contentsActor,
-            Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
-
-        // pack the main GtkNotebook and a spinnerbox in a BinLayout, so that
-        // we can easily bring them front/back
-        this._viewLayout = new Clutter.BinLayout();
-        this._viewActor = new Clutter.Box({ layout_manager: this._viewLayout });
-        this._contentsLayout.set_expand(this._viewActor, true);
-        this._contentsLayout.set_fill(this._viewActor, true, true);
-        this._contentsActor.add_actor(this._viewActor);
-
-        this._notebook = new Gtk.Notebook({ show_tabs: false,
-                                            show_border: false });
-        this._notebook.show();
-        this._notebookActor = new GtkClutter.Actor({ contents: this._notebook,
-                                                     x_align: Clutter.ActorAlign.FILL,
-                                                     x_expand: true,
-                                                     y_align: Clutter.ActorAlign.FILL,
-                                                     y_expand: true });
-        this._viewActor.add_child(this._notebookActor);
-
-        this._spinnerBox = new SpinnerBox();
-        this._viewActor.insert_child_below(this._spinnerBox.actor, null);
-
-        this._errorBox = new ErrorBox();
-        this._viewActor.insert_child_below(this._errorBox.actor,  null);
-
-        this._noResults = new EmptyResultsBox();
-        this._viewLayout.add(this._noResults.actor, Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
-        this._noResults.actor.lower_bottom();
+        this._stack = new Gd.Stack({ visible: true,
+                                     homogeneous: true });
+        this._stackOverlay.add(this._stack);
 
         // create the OSD toolbar for selected items, it's hidden by default
-        this._selectionToolbar = new Selections.SelectionToolbar(this._contentsActor);
-        this._overlayLayout.add(this._selectionToolbar.actor,
-            Clutter.BinAlignment.FIXED, Clutter.BinAlignment.FIXED);
+        this._selectionToolbar = new Selections.SelectionToolbar();
+        this._stackOverlay.add_overlay(this._selectionToolbar.widget);
 
-        // pack the OSD notification actor
-        this._viewActor.add_child(Application.notificationManager.actor);
+        // pack the OSD notification widget
+        this._stackOverlay.add_overlay(Application.notificationManager.widget);
 
         // now create the actual content widgets
         this._view = new View.ViewContainer();
-        this._viewPage = this._notebook.append_page(this._view.widget, null);
+        this._stack.add_named(this._view.widget, 'view');
 
-        this._preview = new Preview.PreviewView(this._overlayLayout);
-        this._previewPage = this._notebook.append_page(this._preview.widget, null);
+        this._preview = new Preview.PreviewView(this._stackOverlay);
+        this._stack.add_named(this._preview.widget, 'preview');
 
-        this._edit = new Edit.EditView(this._overlayLayout);
-        this._editPage = this._notebook.append_page(this._edit.widget, null);
+        this._edit = new Edit.EditView(this._stackOverlay);
+        this._stack.add_named(this._edit.widget, 'edit');
+
+        this._spinnerBox = new SpinnerBox();
+        this._stack.add_named(this._spinnerBox.widget, 'spinner');
+
+        this._errorBox = new ErrorBox();
+        this._stack.add_named(this._errorBox.widget, 'error');
+
+        this._noResults = new EmptyResultsBox();
+        this._stack.add_named(this._noResults.widget, 'no-results');
 
         Application.modeController.connect('window-mode-changed',
                                            Lang.bind(this, this._onWindowModeChanged));
@@ -419,10 +283,11 @@ const Embed = new Lang.Class({
         let queryStatus = Application.trackerController.getQueryStatus();
 
         if (queryStatus) {
-            this._errorBox.moveOut();
-            this._spinnerBox.moveIn();
+            this._spinnerBox.start();
+            this._stack.set_visible_child_name('spinner');
         } else {
-            this._spinnerBox.moveOut();
+            this._spinnerBox.stop();
+            this._stack.set_visible_child_name('view');
         }
     },
 
@@ -432,7 +297,7 @@ const Embed = new Lang.Class({
             this._noResultsChangeId = 0;
         }
 
-        this._noResults.actor.lower_bottom();
+        this._stack.set_visible_child_name('view');
     },
 
     _onItemCountChanged: function() {
@@ -446,7 +311,7 @@ const Embed = new Lang.Class({
                         this._hideNoResultsPage();
                     }));
 
-            this._noResults.actor.raise_top();
+            this._stack.set_visible_child_name('no-results');
         } else {
             this._hideNoResultsPage();
         }
@@ -457,7 +322,7 @@ const Embed = new Lang.Class({
     },
 
     _onFullscreenChanged: function(controller, fullscreen) {
-        this._toolbar.actor.visible = !fullscreen;
+        this._toolbar.widget.visible = !fullscreen;
         this._toolbar.widget.sensitive = !fullscreen;
     },
 
@@ -477,7 +342,7 @@ const Embed = new Lang.Class({
         case WindowMode.WindowMode.NONE:
             break;
          default:
-            throw(new Error('Not handled'))
+            throw(new Error('Not handled'));
             break;
         }
     },
@@ -494,10 +359,23 @@ const Embed = new Lang.Class({
         Application.modeController.setWindowMode(newMode);
     },
 
+    _clearLoadTimer: function() {
+        if (this._loadShowId != 0) {
+            Mainloop.source_remove(this._loadShowId);
+            this._loadShowId = 0;
+        }
+    },
+
     _onLoadStarted: function() {
-        // switch to preview mode, and schedule the spinnerbox to
-        // move in if the document is not loaded by the timeout
-        this._spinnerBox.moveInDelayed(_PDF_LOADER_TIMEOUT);
+        this._clearLoadTimer();
+        this._loadShowId = Mainloop.timeout_add(_PDF_LOADER_TIMEOUT, Lang.bind(this,
+            function() {
+                this._loadShowId = 0;
+
+                this._stack.set_visible_child_name('spinner');
+                this._spinnerBox.start();
+                return false;
+            }));
     },
 
     _onLoadFinished: function(manager, doc, docModel) {
@@ -507,11 +385,14 @@ const Embed = new Lang.Class({
         this._preview.setModel(docModel);
         this._preview.widget.grab_focus();
 
-        this._spinnerBox.moveOut();
+        this._clearLoadTimer();
+        this._spinnerBox.stop();
+        this._stack.set_visible_child_name('preview');
     },
 
     _onLoadError: function(manager, doc, message, exception) {
-        this._spinnerBox.moveOut();
+        this._clearLoadTimer();
+        this._spinnerBox.stop();
         this._setError(message, exception.message);
     },
 
@@ -522,53 +403,43 @@ const Embed = new Lang.Class({
             this._edit.setUri(null);
 
         if (this._toolbar)
-            this._toolbar.actor.destroy();
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
-        this._toolbar = new MainToolbar.OverviewToolbar(this._viewLayout);
-        this._contentsLayout.pack_start = true;
-        this._contentsActor.add_actor(this._toolbar.actor);
-        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
+        this._toolbar = new MainToolbar.OverviewToolbar(this._stackOverlay);
+        this.widget.pack_start(this._toolbar.widget, false, false, 0);
 
-        this._spinnerBox.moveOut();
-        this._errorBox.moveOut();
-
-        this._notebook.set_current_page(this._viewPage);
+        this._spinnerBox.stop();
+        this._stack.set_visible_child_name('view');
     },
 
     _prepareForPreview: function() {
         if (this._edit)
             this._edit.setUri(null);
         if (this._toolbar)
-            this._toolbar.actor.destroy();
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
         this._toolbar = new Preview.PreviewToolbar(this._preview);
-        this._contentsLayout.pack_start = true;
-        this._contentsActor.add_actor(this._toolbar.actor);
-        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
-
-        this._notebook.set_current_page(this._previewPage);
+        this.widget.pack_start(this._toolbar.widget, false, false, 0);
     },
 
     _prepareForEdit: function() {
         if (this._preview)
             this._preview.setModel(null);
         if (this._toolbar)
-            this._toolbar.actor.destroy();
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
         this._toolbar = new Edit.EditToolbar(this._preview);
-        this._contentsLayout.pack_start = true;
-        this._contentsActor.add_actor(this._toolbar.actor);
-        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
+        this.widget.pack_start(this._toolbar.widget, false, false, 0);
 
-        this._notebook.set_current_page(this._editPage);
+        this._stack.set_visible_child_name('edit');
     },
 
     _setError: function(primary, secondary) {
         this._errorBox.update(primary, secondary);
-        this._errorBox.moveIn();
+        this._stack.set_visible_child_name('error');
     },
 
     getMainToolbar: function() {
