@@ -259,7 +259,6 @@ thumbnail_job_completed_cb (EvJobThumbnail *job,
         GdNavBarPrivate *priv = self->priv;
         GdkPixbuf *pixbuf;
         PreviewItem *item;
-        int page;
 
         pixbuf = ev_document_misc_render_thumbnail_with_frame (GTK_WIDGET (self), job->thumbnail);
 
@@ -426,15 +425,11 @@ previews_clear (GdNavBar *self)
 static void
 previews_reload (GdNavBar *self)
 {
-        EvDocumentModel *model;
-
         if (self->priv->document == NULL ||
             self->priv->n_pages <= 0) {
                 return;
         }
 
-        model = self->priv->model;
-
         previews_clear (self);
         previews_create (self);
 }
@@ -682,18 +677,12 @@ hide_preview (GdNavBar *self)
 static void
 show_preview (GdNavBar *self)
 {
-        GdkWindow *parent;
-        GdkWindow *window;
-        int x;
-        int y;
-        int width;
-        int height;
-        int bx;
-        int by;
-        int bwidth;
-        int bheight;
-        int rx;
-        int ry;
+        GdkWindow *window, *toplevel_window;
+        GtkWidget *toplevel;
+        int x, y;
+        int width, height;
+        int bx, by;
+        int tx, ty;
 
         gtk_widget_realize (self->priv->preview_window);
 
@@ -702,14 +691,13 @@ show_preview (GdNavBar *self)
 
         window = gtk_widget_get_window (GTK_WIDGET (self));
         gdk_window_get_position (window, &bx, &by);
-        bwidth = gdk_window_get_width (window);
-        bheight = gdk_window_get_height (window);
 
-        parent = gtk_widget_get_parent_window (GTK_WIDGET (self));
-        gdk_window_get_root_coords (parent, bx, by, &rx, &ry);
+        toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
+        toplevel_window = gtk_widget_get_window (toplevel);
+        gdk_window_get_origin (toplevel_window, &tx, &ty);
 
-        x = rx + bwidth / 2 - width / 2;
-        y = ry - height - 10;
+        x = tx + bx + (gdk_window_get_width (window) - width) / 2;
+        y = ty + by - height - 10;
 
         gtk_window_move (GTK_WINDOW (self->priv->preview_window), x, y);
         gtk_window_present (GTK_WINDOW (self->priv->preview_window));
