@@ -164,91 +164,6 @@ gd_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
 	}
 }
 
-/**
- * gd_embed_image_in_frame: 
- * @source_image:
- * @frame_image_path:
- * @slice_width:
- * @border_width:
- *
- * Returns: (transfer full):
- */
-GdkPixbuf *
-gd_embed_image_in_frame (GdkPixbuf *source_image,
-                         const gchar *frame_image_path,
-                         GtkBorder *slice_width,
-                         GtkBorder *border_width)
-{
-  cairo_surface_t *surface;
-  cairo_t *cr;
-  int source_width, source_height;
-  int dest_width, dest_height;
-  gchar *css_str;
-  GtkCssProvider *provider;
-  GtkStyleContext *context;
-  GError *error = NULL;
-  GdkPixbuf *retval;
-  GtkWidgetPath *path;
- 
-  source_width = gdk_pixbuf_get_width (source_image);
-  source_height = gdk_pixbuf_get_height (source_image);
-
-  dest_width = source_width +  border_width->left + border_width->right;
-  dest_height = source_height + border_width->top + border_width->bottom;
-
-  css_str = g_strdup_printf (".embedded-image { border-image: url(\"%s\") %d %d %d %d / %dpx %dpx %dpx %dpx }",
-                             frame_image_path, 
-                             slice_width->top, slice_width->right, slice_width->bottom, slice_width->left,
-                             border_width->top, border_width->right, border_width->bottom, border_width->left);
-  provider = gtk_css_provider_new ();
-  gtk_css_provider_load_from_data (provider, css_str, -1, &error);
-
-  if (error != NULL) 
-    {
-      g_warning ("Unable to create the thumbnail frame image: %s", error->message);
-      g_error_free (error);
-      g_free (css_str);
-
-      return g_object_ref (source_image);
-    }
-
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, dest_width, dest_height);
-  cr = cairo_create (surface);
-
-  context = gtk_style_context_new ();
-  path = gtk_widget_path_new ();
-  gtk_widget_path_append_type (path, GTK_TYPE_ICON_VIEW);
-
-  gtk_style_context_set_path (context, path);
-  gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), 600);
-
-  gtk_style_context_save (context);
-  gtk_style_context_add_class (context, "embedded-image");
-
-  gtk_render_frame (context, cr,
-                    0, 0,
-                    dest_width, dest_height);
-
-  gtk_style_context_restore (context);
-
-  gtk_render_icon (context, cr,
-                   source_image,
-                   border_width->left, border_width->top);
-
-  retval = gdk_pixbuf_get_from_surface (surface,
-                                        0, 0, dest_width, dest_height);
-
-  cairo_surface_destroy (surface);
-  cairo_destroy (cr);
-
-  gtk_widget_path_unref (path);
-  g_object_unref (provider);
-  g_object_unref (context);
-  g_free (css_str);
-
-  return retval;
-}
-
 const char *
 gd_filename_get_extension_offset (const char *filename)
 {
