@@ -19,12 +19,10 @@
  *
  */
 
-const Clutter = imports.gi.Clutter;
 const Gd = imports.gi.Gd;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 const Pango = imports.gi.Pango;
 
 const Gettext = imports.gettext;
@@ -35,7 +33,6 @@ const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const Searchbar = imports.searchbar;
-const Tweener = imports.util.tweener;
 
 const MainToolbar = new Lang.Class({
     Name: 'MainToolbar',
@@ -43,24 +40,16 @@ const MainToolbar = new Lang.Class({
     _init: function() {
         this._model = null;
 
-        this.widget = new Gd.MainToolbar({ icon_size: Gtk.IconSize.MENU });
-        this.widget.get_style_context().add_class(Gtk.STYLE_CLASS_MENUBAR);
+        this.widget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
         this.widget.show();
 
-        this.layout = new Clutter.BoxLayout({ vertical: true });
-        this.actor = new Clutter.Actor({ layout_manager: this.layout });
+        this.toolbar = new Gd.HeaderBar();
+        this.widget.add(this.toolbar);
+        this.toolbar.show();
 
-        this.toolbarActor = new GtkClutter.Actor({ contents: this.widget });
-
-        this.layout.pack(this.toolbarActor,
-                         false, true, false,
-                         Clutter.BoxAlignment.CENTER, Clutter.BoxAlignment.START);
-
-        this.createSearchbar();
-    },
-
-    createSearchbar: function() {
-        log('Error: MainToolbar subclasses must implement createSearchbar');
+        this._searchbar = this.createSearchbar();
+        if (this._searchbar)
+            this.widget.add(this._searchbar.widget);
     },
 
     handleEvent: function(event) {
@@ -69,9 +58,10 @@ const MainToolbar = new Lang.Class({
     },
 
     addSearchButton: function() {
-        let searchButton =
-            this.widget.add_toggle('edit-find-symbolic', _("Search"), false);
-        searchButton.action_name = 'app.search';
+        let searchButton = new Gd.HeaderToggleButton({ symbolic_icon_name: 'edit-find-symbolic',
+                                                       label: _("Search"),
+                                                       action_name: 'app.search' });
+        this.toolbar.pack_end(searchButton);
     }
 });
 
@@ -79,11 +69,12 @@ const OverviewToolbar = new Lang.Class({
     Name: 'OverviewToolbar',
     Extends: MainToolbar,
 
-    _init: function(viewLayout) {
-        this._viewLayout = viewLayout;
+    _init: function(overlay) {
+        this._overlay = overlay;
         this._collBackButton = null;
         this._collectionId = 0;
         this._selectionChangedId = 0;
+        this._selectionMenu = null;
 
         this.parent();
 
@@ -135,7 +126,6 @@ const OverviewToolbar = new Lang.Class({
         let selectionMode = Application.selectionController.getSelectionMode();
         let activeCollection = Application.collectionManager.getActiveItem();
         let primary = null;
-        let detail = null;
 
         if (!selectionMode) {
             if (activeCollection) {
@@ -145,60 +135,51 @@ const OverviewToolbar = new Lang.Class({
 
                 if (string == '') {
                     let searchType = Application.searchTypeManager.getActiveItem();
-                    let searchSource = Application.sourceManager.getActiveItem();
 
                     if (searchType.id != 'all')
                         primary = searchType.name;
-
-                    if (searchSource.id != 'all')
-                        detail = searchSource.name;
                 } else {
-                    let searchMatch = Application.searchMatchManager.getActiveItem();
-
                     primary = _("Results for “%s”").format(string);
-                    if (searchMatch.id == 'title')
-                        detail = _("filtered by title");
-                    else if (searchMatch.id == 'author')
-                        detail = _("filtered by author");
                 }
             }
         } else {
             let length = Application.selectionController.getSelection().length;
+            let label = null;
 
             if (length == 0)
-                detail = _("Click on items to select them");
+                label = _("Click on items to select them");
             else
-                detail = Gettext.ngettext("%d selected",
-                                          "%d selected",
-                                          length).format(length);
+                label = Gettext.ngettext("%d selected",
+                                         "%d selected",
+                                         length).format(length);
 
-            if (activeCollection) {
-                primary = activeCollection.name;
-            } else if (length != 0) {
-                primary = detail;
-                detail = null;
-            }
+            if (activeCollection)
+                primary = ("<b>%s</b>  (%s)").format(activeCollection.name, label);
+            else
+                primary = label;
         }
 
-        if (detail)
-            detail = '(' + detail + ')';
-
-        this.widget.set_labels(primary, detail);
+        if (this._selectionMenu)
+            this._selectionMenu.set_label(primary);
+        else
+            this.toolbar.set_title(primary);
     },
 
     _populateForSelectionMode: function() {
-        this.widget.get_style_context().add_class('selection-mode');
-        this.widget.reset_style();
+        this.toolbar.get_style_context().add_class('selection-mode');
 
         this.addSearchButton();
 
         let builder = new Gtk.Builder();
         builder.add_from_resource('/org/gnome/documents/selection-menu.ui');
         let selectionMenu = builder.get_object('selection-menu');
-        this.widget.set_labels_menu(selectionMenu);
+        this._selectionMenu = new Gd.HeaderMenuButton({ menu_model: selectionMenu,
+                                                        use_markup: true });
+        this._selectionMenu.get_style_context().add_class('selection-menu');
+        this.toolbar.set_custom_title(this._selectionMenu);
 
-        let selectionButton =
-            this.widget.add_button(null, _("Done"), false);
+        let selectionButton = new Gd.HeaderSimpleButton({ label: _("Done") });
+        this.toolbar.pack_end(selectionButton);
         selectionButton.get_style_context().add_class('suggested-action');
         selectionButton.connect('clicked', Lang.bind(this,
             function() {
@@ -215,8 +196,10 @@ const OverviewToolbar = new Lang.Class({
         let item = Application.collectionManager.getActiveItem();
 
         if (item && !this._collBackButton) {
-            this._collBackButton =
-                this.widget.add_button('go-previous-symbolic', _("Back"), true);
+            this._collBackButton = new Gd.HeaderSimpleButton({ symbolic_icon_name: 'go-previous-symbolic',
+                                                               label: _("Back"),
+                                                               visible: true });
+            this.toolbar.pack_start(this._collBackButton);
             this._collBackButton.connect('clicked', Lang.bind(this,
                 function() {
                     Application.documentManager.activatePreviousCollection();
@@ -237,8 +220,9 @@ const OverviewToolbar = new Lang.Class({
         this._checkCollectionBackButton();
         this.addSearchButton();
 
-        let selectionButton =
-            this.widget.add_button('object-select-symbolic', _("Select Items"), false);
+        let selectionButton = new Gd.HeaderSimpleButton({ symbolic_icon_name: 'object-select-symbolic',
+                                                          label: _("Select Items") });
+        this.toolbar.pack_end(selectionButton);
         selectionButton.connect('clicked', Lang.bind(this,
             function() {
                 Application.selectionController.setSelectionMode(true);
@@ -252,7 +236,8 @@ const OverviewToolbar = new Lang.Class({
 
     _clearStateData: function() {
         this._collBackButton = null;
-        this.widget.set_labels_menu(null);
+        this._selectionMenu = null;
+        this.toolbar.set_custom_title(null);
 
         if (this._collectionId != 0) {
             Application.collectionManager.disconnect(this._collectionId);
@@ -268,9 +253,9 @@ const OverviewToolbar = new Lang.Class({
     _clearToolbar: function() {
         this._clearStateData();
 
-        this.widget.get_style_context().remove_class('selection-mode');
-        this.widget.reset_style();
-        this.widget.clear();
+        this.toolbar.get_style_context().remove_class('selection-mode');
+        let children = this.toolbar.get_children();
+        children.forEach(function(child) { child.destroy(); });
     },
 
     _resetToolbarMode: function() {
@@ -283,7 +268,7 @@ const OverviewToolbar = new Lang.Class({
             this._populateForOverview();
 
         this._setToolbarTitle();
-        this.widget.show_all();
+        this.toolbar.show_all();
 
         if (Application.searchController.getString() != '')
             Application.application.change_action_state('search', GLib.Variant.new('b', true));
@@ -292,12 +277,8 @@ const OverviewToolbar = new Lang.Class({
     createSearchbar: function() {
         // create the dropdown for the search bar, it's hidden by default
         let dropdown = new Searchbar.Dropdown();
-        this._viewLayout.add(dropdown.actor,
-            Clutter.BinAlignment.CENTER, Clutter.BinAlignment.FIXED);
+        this._overlay.add_overlay(dropdown.widget);
 
-        this._searchbar = new Searchbar.OverviewSearchbar(dropdown);
-        this.layout.pack_start = false;
-        this.layout.pack(this._searchbar.actor, false, true, false,
-                         Clutter.BoxAlignment.CENTER, Clutter.BoxAlignment.START);
+        return new Searchbar.OverviewSearchbar(dropdown);
     }
 });
