@@ -19,11 +19,9 @@
  *
  */
 
-const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
@@ -159,7 +157,7 @@ const MainWindow = new Lang.Class({
         case WindowMode.WindowMode.EDIT:
             return false;
         default:
-            throw(new Error('Not handled'))
+            throw(new Error('Not handled'));
             break;
         }
 
@@ -174,8 +172,9 @@ const MainWindow = new Lang.Class({
 
         if (keyval == Gdk.KEY_Escape) {
             let preview = this._embed.getPreview();
+            let model = preview.getModel();
 
-            if (preview.controlsVisible)
+            if (preview.controlsVisible && (model != null))
                 preview.controlsVisible = false;
             else if (fullscreen)
                 Application.documentManager.setActiveItem(null);
