@@ -19,34 +19,36 @@
  *
  */
 
-const Clutter = imports.gi.Clutter;
+const EvDocument = imports.gi.EvinceDocument;
 const EvView = imports.gi.EvinceView;
+const Gd = imports.gi.Gd;
 const GdPrivate = imports.gi.GdPrivate;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 const _ = imports.gettext.gettext;
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Signals = imports.signals;
+const Tweener = imports.tweener.tweener;
 
 const Application = imports.application;
-const Tweener = imports.util.tweener;
 const MainToolbar = imports.mainToolbar;
 const Places = imports.places;
 const Searchbar = imports.searchbar;
 const Utils = imports.utils;
 const View = imports.view;
+const WindowMode = imports.windowMode;
+const Presentation = imports.presentation;
 
 const _FULLSCREEN_TOOLBAR_TIMEOUT = 2; // seconds
 
 const PreviewView = new Lang.Class({
     Name: 'PreviewView',
 
-    _init: function(overlayLayout) {
+    _init: function(overlay) {
         this._model = null;
         this._jobFind = null;
         this._controlsFlipId = 0;
@@ -54,15 +56,17 @@ const PreviewView = new Lang.Class({
         this._pageChanged = false;
         this._viewSelectionChanged = false;
         this._fsToolbar = null;
-        this._overlayLayout = overlayLayout;
+        this._overlay = overlay;
         this._lastSearch = '';
+        this._loadError = false;
 
         Application.modeController.connect('fullscreen-changed', Lang.bind(this,
             this._onFullscreenChanged));
+        Application.modeController.connect('window-mode-changed', Lang.bind(this,
+            this._onWindowModeChanged));
 
         this.widget = new Gtk.ScrolledWindow({ hexpand: true,
-                                               vexpand: true,
-                                               shadow_type: Gtk.ShadowType.IN });
+                                               vexpand: true });
         this.widget.get_style_context().add_class('documents-scrolledwin');
         this.widget.get_hscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
         this.widget.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
@@ -72,15 +76,18 @@ const PreviewView = new Lang.Class({
         this._createView();
 
         // create page nav bar
-        this._navBar = new PreviewNav(this._model);
-        this._overlayLayout.add(this._navBar.actor,
-            Clutter.BinAlignment.FILL, Clutter.BinAlignment.END);
+        this._navBar = new PreviewNavBar(this._model);
+        this._overlay.add_overlay(this._navBar.widget);
+
+        // create page nav buttons
+        this._navButtons = new PreviewNavButtons(this._model, this._overlay);
 
         this.widget.show_all();
 
         Application.application.connect('action-state-changed::bookmark-page',
             Lang.bind(this, this._onActionStateChanged));
-        this._onActionStateChanged(Application.application, 'bookmark-page', Application.application.get_action_state('bookmark-page'));
+        this._onActionStateChanged(Application.application, 'bookmark-page',
+            Application.application.get_action_state('bookmark-page'));
 
         this._zoomIn = Application.application.lookup_action('zoom-in');
         this._zoomIn.connect('activate', Lang.bind(this,
@@ -120,6 +127,10 @@ const PreviewView = new Lang.Class({
         let showPlaces = Application.application.lookup_action('places');
         showPlaces.connect('activate', Lang.bind(this, this._showPlaces));
 
+        this._togglePresentation = Application.application.lookup_action('present-current');
+        Application.application.connect('action-state-changed::present-current',
+            Lang.bind(this, this._onPresentStateChanged));
+
         Application.documentManager.connect('load-started',
                                             Lang.bind(this, this._onLoadStarted));
         Application.documentManager.connect('load-finished',
@@ -128,22 +139,28 @@ const PreviewView = new Lang.Class({
                                             Lang.bind(this, this._onLoadError));
     },
 
-   _onLoadStarted: function() {
+    _onLoadStarted: function() {
         this._showPlaces.enabled = false;
     },
 
     _onLoadFinished: function(manager, doc, docModel) {
         this._showPlaces.enabled = true;
+        this._togglePresentation.enabled = true;
 
         if (!Application.documentManager.metadata)
             return;
 
+        this._navButtons.show();
+
         this._bookmarks = new GdPrivate.Bookmarks({ metadata: Application.documentManager.metadata });
     },
 
     _onLoadError: function(manager, doc, message, exception) {
         this._controlsVisible = true;
+
+        this._loadError = true;
         this._syncControlsVisible();
+        this._loadError = false;
     },
 
     _onActionStateChanged: function(source, actionName, state) {
@@ -159,6 +176,16 @@ const PreviewView = new Lang.Class({
             this._bookmarks.remove(bookmark);
     },
 
+    _onPresentStateChanged: function(source, actionName, state) {
+        if (!this._model)
+            return;
+
+        if (state.get_boolean())
+            this._promptPresentation();
+        else
+            this._hidePresentation();
+    },
+
     _onPageChanged: function() {
         this._pageChanged = true;
 
@@ -177,12 +204,102 @@ const PreviewView = new Lang.Class({
             }));
     },
 
+    _hidePresentation: function() {
+        if (this._presentation) {
+            this._presentation.close();
+            this._presentation = null;
+        }
+
+        Application.application.change_action_state('present-current', GLib.Variant.new('b', false));
+    },
+
+    _showPresentation: function(output) {
+        this._presentation = new Presentation.PresentationWindow(this._model);
+        this._presentation.window.connect('destroy', Lang.bind(this, this._hidePresentation));
+        if (output)
+            this._presentation.setOutput(output);
+    },
+
+    _promptPresentation: function() {
+        let outputs = new Presentation.PresentationOutputs();
+        if (outputs.list.length < 2) {
+            this._showPresentation();
+        } else {
+            let chooser = new Presentation.PresentationOutputChooser(outputs);
+            chooser.connect('output-activated', Lang.bind(this,
+                function(chooser, output) {
+                    if (output) {
+                        this._showPresentation(output);
+                    } else {
+                        this._hidePresentation();
+                    }
+                }));
+
+        }
+    },
+
     _onViewSelectionChanged: function() {
         this._viewSelectionChanged = true;
         if (!this.view.get_has_selection())
             this._cancelControlsFlip();
     },
 
+    _uriRewrite: function(uri) {
+        if (uri.substring(0, 3) != 'www.') {
+            /* Prepending "http://" when the url is a webpage (starts with
+             * "www.").
+             */
+            uri = 'http://' + uri;
+        } else {
+            /* Or treating as a file, otherwise.
+             * http://en.wikipedia.org/wiki/File_URI_scheme
+             */
+            let doc = Application.documentManager.getActiveItem();
+            let file = Gio.file_new_for_uri(doc.uri);
+            let parent = file.get_parent();
+
+            if (parent)
+                uri = parent.get_uri() + uri;
+            else
+                uri = 'file:///' + uri;
+        }
+
+        return uri;
+    },
+
+    _launchExternalUri: function(widget, action) {
+        let uri = action.get_uri();
+        let screen = widget.get_screen();
+        let context = screen.get_display().get_app_launch_context();
+
+        context.set_screen(screen);
+        context.set_timestamp(Gtk.get_current_event_time());
+
+        if (uri.indexOf('://') == -1 && uri.substring(0, 6) != 'mailto:')
+            /* We are only interested in treat URLs (ignoring URN and Mailto
+             * schemes), which have this syntax scheme:
+             * scheme://domain:port/path?query_string#fragment_id
+             *
+             * So, if the url is bad formed (doesn't contain "://"), we need to
+             * rewrite it.
+             *
+             * An example of URL, URN and Mailto schemes can be found in:
+             * http://en.wikipedia.org/wiki/URI_scheme#Examples
+             */
+            uri = this._uriRewrite(uri);
+
+        try {
+            Gio.AppInfo.launch_default_for_uri(uri, context)
+        } catch (e) {
+            log('Unable to open external link: ' + e.message);
+        }
+    },
+
+    _handleExternalLink: function(widget, action) {
+        if (action.type == EvDocument.LinkActionType.EXTERNAL_URI)
+            this._launchExternalUri(widget, action);
+    },
+
     _onCanZoomInChanged: function() {
         this._zoomIn.enabled = this.view.can_zoom_in;
     },
@@ -208,13 +325,16 @@ const PreviewView = new Lang.Class({
             this._onKeyPressEvent));
         this.view.connect('selection-changed', Lang.bind(this,
             this._onViewSelectionChanged));
+        this.view.connect('external-link', Lang.bind(this,
+            this._handleExternalLink));
     },
 
     _syncControlsVisible: function() {
         if (this._controlsVisible) {
             if (this._fsToolbar)
                 this._fsToolbar.show();
-            this._navBar.show();
+            if (!this._loadError)
+                this._navBar.show();
         } else {
             if (this._fsToolbar)
                 this._fsToolbar.hide();
@@ -222,6 +342,15 @@ const PreviewView = new Lang.Class({
         }
     },
 
+    _onWindowModeChanged: function() {
+        let windowMode = Application.modeController.getWindowMode();
+        if (windowMode != WindowMode.WindowMode.PREVIEW) {
+            this.controlsVisible = false;
+            this._hidePresentation();
+            this._navButtons.hide();
+        }
+    },
+
     _onFullscreenChanged: function() {
         let fullscreen = Application.modeController.getFullscreen();
 
@@ -229,15 +358,14 @@ const PreviewView = new Lang.Class({
             // create fullscreen toolbar (hidden by default)
             this._fsToolbar = new PreviewFullscreenToolbar(this);
             this._fsToolbar.setModel(this._model);
-            this._overlayLayout.add(this._fsToolbar.actor,
-                Clutter.BinAlignment.FILL, Clutter.BinAlignment.START);
+            this._overlay.add_overlay(this._fsToolbar.revealer);
 
             this._fsToolbar.connect('show-controls', Lang.bind(this,
                 function() {
                     this.controlsVisible = true;
                 }));
         } else {
-            this._fsToolbar.actor.destroy();
+            this._fsToolbar.revealer.destroy();
             this._fsToolbar = null;
         }
 
@@ -363,13 +491,13 @@ const PreviewView = new Lang.Class({
             this._jobFind = null;
         }
 
+        this._lastSearch = str;
+
         if (!str) {
             this.view.queue_draw();
             return;
         }
 
-        this._lastSearch = str;
-
         let evDoc = this._model.get_document();
         this._jobFind = EvView.JobFind.new(evDoc, this._model.get_page(), evDoc.get_n_pages(),
                                            str, false);
@@ -390,7 +518,6 @@ const PreviewView = new Lang.Class({
             return;
 
         if (this.view) {
-            this.view.destroy();
             this.controlsVisible = false;
             this._lastSearch = '';
         }
@@ -398,9 +525,9 @@ const PreviewView = new Lang.Class({
         this._model = model;
 
         if (this._model) {
-            this._createView();
             this.view.set_model(this._model);
             this._navBar.setModel(model);
+            this._navButtons.setModel(model);
             this._model.connect('page-changed', Lang.bind(this, this._onPageChanged));
         }
     },
@@ -419,14 +546,17 @@ const PreviewView = new Lang.Class({
 });
 Signals.addSignalMethods(PreviewView.prototype);
 
-const _PREVIEW_NAVBAR_MARGIN = 40;
+const _PREVIEW_NAVBAR_MARGIN = 30;
 
-const PreviewNav = new Lang.Class({
-    Name: 'PreviewNav',
+const PreviewNavBar = new Lang.Class({
+    Name: 'PreviewNavBar',
 
     _init: function(model) {
         this._model = model;
-        this.widget = new GdPrivate.NavBar({ document_model: model });
+        this.widget = new GdPrivate.NavBar({ document_model: model,
+                                             margin: _PREVIEW_NAVBAR_MARGIN,
+                                             valign: Gtk.Align.END,
+                                             opacity: 0 });
         this.widget.get_style_context().add_class('osd');
 
         let button = new Gtk.Button({ action_name: 'app.places',
@@ -443,17 +573,6 @@ const PreviewNav = new Lang.Class({
                                         valign: Gtk.Align.CENTER
                                       });
         buttonArea.pack_start(button, false, false, 0);
-
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            visible: false,
-                                            margin_top: _PREVIEW_NAVBAR_MARGIN,
-                                            margin_bottom: _PREVIEW_NAVBAR_MARGIN,
-                                            margin_left: _PREVIEW_NAVBAR_MARGIN,
-                                            margin_right: _PREVIEW_NAVBAR_MARGIN,
-                                            opacity: 0 });
-        Utils.alphaGtkWidget(this.actor.get_widget());
-
-        this.widget.show_all();
     },
 
     setModel: function(model) {
@@ -467,26 +586,188 @@ const PreviewNav = new Lang.Class({
         if (!this._model)
             return;
 
-        this.actor.show();
-
-        Tweener.addTween(this.actor,
-            { opacity: 255,
-              time: 0.30,
-              transition: 'easeOutQuad' });
+        this.widget.show_all();
+        Tweener.addTween(this.widget, { opacity: 1,
+                                        time: 0.30,
+                                        transition: 'easeOutQuad' });
     },
 
     hide: function() {
-        Tweener.addTween(this.actor,
-            { opacity: 0,
-              time: 0.30,
-              transition: 'easeOutQuad',
-              onComplete: function() {
-                  this.actor.hide();
-              },
-              onCompleteScope: this });
+        Tweener.addTween(this.widget, { opacity: 0,
+                                        time: 0.30,
+                                        transition: 'easeOutQuad',
+                                        onComplete: function() {
+                                            this.widget.hide();
+                                        },
+                                        onCompleteScope: this });
     }
 });
 
+const _AUTO_HIDE_TIMEOUT = 2;
+const PreviewNavButtons = new Lang.Class({
+    Name: 'PreviewNavButtons',
+
+    _init: function(model, overlay) {
+        this._model = model;
+        this._overlay = overlay;
+        this._visible = false;
+        this._pageChangedId = 0;
+        this._autoHideId = 0;
+        this._motionId = 0;
+        this._hover = false;
+
+        this.prev_widget = new Gtk.Button({ child: new Gtk.Image ({ icon_name: 'go-previous-symbolic',
+                                                                    pixel_size: 16 }),
+                                            margin_left: _PREVIEW_NAVBAR_MARGIN,
+                                            halign: Gtk.Align.START,
+                                            valign: Gtk.Align.CENTER });
+        this.prev_widget.get_style_context().add_class('osd');
+        this._overlay.add_overlay(this.prev_widget);
+        this.prev_widget.connect('clicked', Lang.bind(this, this._onPrevClicked));
+        this.prev_widget.connect('enter-notify-event', Lang.bind(this, this._onEnterNotify));
+        this.prev_widget.connect('leave-notify-event', Lang.bind(this, this._onLeaveNotify));
+
+        this.next_widget = new Gtk.Button({ child: new Gtk.Image ({ icon_name: 'go-next-symbolic',
+                                                                    pixel_size: 16 }),
+                                            margin_right: _PREVIEW_NAVBAR_MARGIN,
+                                            halign: Gtk.Align.END,
+                                            valign: Gtk.Align.CENTER });
+        this.next_widget.get_style_context().add_class('osd');
+        this._overlay.add_overlay(this.next_widget);
+        this.next_widget.connect('clicked', Lang.bind(this, this._onNextClicked));
+        this.next_widget.connect('enter-notify-event', Lang.bind(this, this._onEnterNotify));
+        this.next_widget.connect('leave-notify-event', Lang.bind(this, this._onLeaveNotify));
+
+        this._overlay.connect('motion-notify-event', Lang.bind(this, this._onMotion));
+
+    },
+
+    _onEnterNotify: function() {
+        this._hover = true;
+        this._unqueueAutoHide();
+        return false;
+    },
+
+    _onLeaveNotify: function() {
+        this._hover = false;
+        this._queueAutoHide();
+        return false;
+    },
+
+    _motionTimeout: function() {
+        this._motionId = 0;
+        this._updateVisibility();
+        return false;
+    },
+
+    _onMotion: function() {
+        if (this._motionId != 0) {
+            return false;
+        }
+
+        this._motionId = Mainloop.idle_add(Lang.bind(this, this._motionTimeout));
+        return false;
+    },
+
+    _onPrevClicked: function() {
+        if (this._model.page > 0)
+            this._model.page--;
+    },
+
+    _onNextClicked: function() {
+        let doc = this._model.document;
+        if (doc.get_n_pages() > this._model.page + 1)
+            this._model.page++;
+    },
+
+    _autoHide: function() {
+        this._fadeOutButton(this.prev_widget);
+        this._fadeOutButton(this.next_widget);
+        this._autoHideId = 0;
+        return false;
+    },
+
+    _unqueueAutoHide: function() {
+        if (this._autoHideId == 0)
+            return;
+
+        Mainloop.source_remove(this._autoHideId);
+        this._autoHideId = 0;
+    },
+
+    _queueAutoHide: function() {
+        this._unqueueAutoHide();
+        this._autoHideId = Mainloop.timeout_add_seconds(_AUTO_HIDE_TIMEOUT, Lang.bind(this, this._autoHide));
+    },
+
+    _updateVisibility: function() {
+        if (!this._model || !this._visible) {
+            this._fadeOutButton(this.prev_widget);
+            this._fadeOutButton(this.next_widget);
+            return;
+        }
+
+        if (this._model.page > 0)
+            this._fadeInButton(this.prev_widget);
+        else
+            this._fadeOutButton(this.prev_widget);
+
+        let doc = this._model.document;
+        if (doc.get_n_pages() > this._model.page + 1)
+            this._fadeInButton(this.next_widget);
+        else
+            this._fadeOutButton(this.next_widget);
+
+        if (!this._hover)
+            this._queueAutoHide();
+    },
+
+    setModel: function(model) {
+        if (this._pageChangedId != 0) {
+            this._model.disconnect(this._pageChangedId);
+            this._pageChangedId = 0;
+        }
+
+        this._model = model;
+
+        if (this._model)
+            this._pageChangedId = this._model.connect('page-changed', Lang.bind(this, this._updateVisibility));
+
+        this._updateVisibility();
+    },
+
+    _fadeInButton: function(widget) {
+        if (!this._model)
+            return;
+        widget.show_all();
+        Tweener.addTween(widget, { opacity: 1,
+                                   time: 0.30,
+                                   transition: 'easeOutQuad' });
+    },
+
+    _fadeOutButton: function(widget) {
+        Tweener.addTween(widget, { opacity: 0,
+                                   time: 0.30,
+                                   transition: 'easeOutQuad',
+                                   onComplete: function() {
+                                       widget.hide();
+                                   },
+                                   onCompleteScope: this });
+    },
+
+    show: function() {
+        this._visible = true;
+        this._fadeInButton(this.prev_widget);
+        this._fadeInButton(this.next_widget);
+    },
+
+    hide: function() {
+        this._visible = false;
+        this._fadeOutButton(this.prev_widget);
+        this._fadeOutButton(this.next_widget);
+    },
+});
+
 const PreviewToolbar = new Lang.Class({
     Name: 'PreviewToolbar',
     Extends: MainToolbar.MainToolbar,
@@ -496,15 +777,20 @@ const PreviewToolbar = new Lang.Class({
 
         this.parent();
 
+        this._searchAction = Application.application.lookup_action('search');
+        this._gearMenu = Application.application.lookup_action('gear-menu');
+
         // back button, on the left of the toolbar
         let iconName =
-            (this.widget.get_direction() == Gtk.TextDirection.RTL) ?
+            (this.toolbar.get_direction() == Gtk.TextDirection.RTL) ?
             'go-next-symbolic' : 'go-previous-symbolic';
-        let backButton =
-            this.widget.add_button(iconName, _("Back"), true);
+        let backButton = new Gd.HeaderSimpleButton({ symbolic_icon_name: iconName,
+                                                     label: _("Back") });
+        this.toolbar.pack_start(backButton);
         backButton.connect('clicked', Lang.bind(this,
             function() {
                 Application.documentManager.setActiveItem(null);
+                this._searchAction.enabled = true;
             }));
 
         // search button, on the right of the toolbar
@@ -512,12 +798,36 @@ const PreviewToolbar = new Lang.Class({
 
         // menu button, on the right of the toolbar
         let previewMenu = this._getPreviewMenu();
-        let menuButton = this.widget.add_menu('emblem-system-symbolic', null, false);
-        menuButton.set_menu_model(previewMenu);
-        menuButton.set_action_name('app.gear-menu');
+        let menuButton = new Gd.HeaderMenuButton({ menu_model: previewMenu,
+                                                   action_name: 'app.gear-menu',
+                                                   symbolic_icon_name: 'emblem-system-symbolic' });
+        this.toolbar.pack_end(menuButton);
 
         this._setToolbarTitle();
-        this.widget.show_all();
+        this.toolbar.show_all();
+
+        let loadStartId =
+            Application.documentManager.connect('load-started',
+                Lang.bind(this, this._onLoadStarted));
+        let loadFinishId =
+            Application.documentManager.connect('load-finished',
+                Lang.bind(this, this._onLoadFinished));
+
+        this.widget.connect('destroy',
+            function() {
+                Application.documentManager.disconnect(loadStartId);
+                Application.documentManager.disconnect(loadFinishId);
+            });
+    },
+
+    _onLoadStarted: function() {
+        this._gearMenu.enabled = false;
+        this._searchAction.enabled = false;
+    },
+
+    _onLoadFinished: function() {
+        this._gearMenu.enabled = true;
+        this._searchAction.enabled = true;
     },
 
     _getPreviewMenu: function() {
@@ -536,10 +846,7 @@ const PreviewToolbar = new Lang.Class({
     },
 
     createSearchbar: function() {
-        this._searchbar = new PreviewSearchbar(this._previewView);
-        this.layout.pack_start = false;
-        this.layout.pack(this._searchbar.actor, false, true, false,
-                         Clutter.BoxAlignment.CENTER, Clutter.BoxAlignment.START);
+        return new PreviewSearchbar(this._previewView);
     },
 
     _setToolbarTitle: function() {
@@ -549,7 +856,7 @@ const PreviewToolbar = new Lang.Class({
         if (doc)
             primary = doc.name;
 
-        this.widget.set_labels(primary, null);
+        this.toolbar.set_title(primary);
     },
 
     setModel: function(model) {
@@ -633,7 +940,9 @@ const PreviewSearchbar = new Lang.Class({
     hide: function() {
         this._previewView.view.find_set_highlight_search(false);
 
+        this.searchChangeBlocked = true;
         this.parent();
+        this.searchChangeBlocked = false;
     }
 });
 
@@ -644,7 +953,9 @@ const PreviewFullscreenToolbar = new Lang.Class({
     _init: function(previewView) {
         this.parent(previewView);
 
-        this.actor.translation_y = -(this.widget.get_preferred_height()[1]);
+        this.revealer = new Gd.Revealer({ valign: Gtk.Align.START });
+        this.revealer.add(this.widget);
+        this.revealer.show();
 
         // make controls show when a toolbar action is activated in fullscreen
         let actionNames = ['gear-menu', 'search'];
@@ -672,31 +983,13 @@ const PreviewFullscreenToolbar = new Lang.Class({
             }));
     },
 
-    handleEvent: function(event) {
-        let res = this.parent(event);
-        if (res)
-            this.emit('search-event-handled');
-    },
-
     show: function() {
-        this.actor.show();
-        Tweener.addTween(this.actor,
-                         { translation_y: 0,
-                           time: 0.20,
-                           transition: 'easeInQuad' });
+        this.revealer.set_reveal_child(true);
     },
 
     hide: function() {
-        Tweener.addTween(this.actor,
-                         { translation_y: -(this.widget.get_preferred_height()[1]),
-                           time: 0.20,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this,
-                               function() {
-                                   this.actor.hide();
-                                   Application.application.change_action_state('search', GLib.Variant.new('b', false));
-                               })
-                         });
+        this.revealer.set_reveal_child(false);
+        Application.application.change_action_state('search', GLib.Variant.new('b', false));
     }
 });
 Signals.addSignalMethods(PreviewFullscreenToolbar.prototype);
