@@ -23,7 +23,6 @@ const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
-const GtkClutter = imports.gi.GtkClutter;
 const Tracker = imports.gi.Tracker;
 const _ = imports.gettext.gettext;
 
@@ -33,7 +32,7 @@ const Signals = imports.signals;
 
 const Application = imports.application;
 const Manager = imports.manager;
-const Tweener = imports.util.tweener;
+const Tweener = imports.tweener.tweener;
 const Utils = imports.utils;
 
 const _SEARCH_ENTRY_TIMEOUT = 200;
@@ -45,14 +44,15 @@ const Searchbar = new Lang.Class({
         this._searchEntryTimeout = 0;
         this._searchTypeId = 0;
         this._searchMatchId = 0;
+        this.searchChangeBlocked = false;
 
         this._in = false;
 
-        this.widget = new Gtk.Toolbar();
-        this.widget.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
+        this.widget = new Gd.Revealer();
 
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            height: 0 });
+        let toolbar = new Gtk.Toolbar();
+        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
+        this.widget.add(toolbar);
 
         // subclasses will create this._searchEntry and this._searchContainer
         // GtkWidgets
@@ -61,7 +61,7 @@ const Searchbar = new Lang.Class({
         let item = new Gtk.ToolItem();
         item.set_expand(true);
         item.add(this._searchContainer);
-        this.widget.insert(item, 0);
+        toolbar.insert(item, 0);
 
         this._searchEntry.connect('key-press-event', Lang.bind(this,
             function(widget, event) {
@@ -82,6 +82,9 @@ const Searchbar = new Lang.Class({
                     this._searchEntryTimeout = 0;
                 }
 
+                if (this.searchChangeBlocked)
+                    return;
+
                 this._searchEntryTimeout = Mainloop.timeout_add(_SEARCH_ENTRY_TIMEOUT, Lang.bind(this,
                     function() {
                         this._searchEntryTimeout = 0;
@@ -196,31 +199,18 @@ const Searchbar = new Lang.Class({
 
     show: function() {
         let eventDevice = Gtk.get_current_event_device();
-        this.widget.show_all();
+        this.widget.set_reveal_child(true);
+        this._in = true;
 
-        Tweener.addTween(this.actor, { height: this.widget.get_preferred_height()[1],
-                                       time: 0.20,
-                                       transition: 'easeOutQuad',
-                                       onComplete: function() {
-                                           this._in = true;
-                                           if (eventDevice)
-                                               Gd.entry_focus_hack(this._searchEntry, eventDevice);
-                                       },
-                                       onCompleteScope: this });
+        if (eventDevice)
+            Gd.entry_focus_hack(this._searchEntry, eventDevice);
     },
 
     hide: function() {
-        Tweener.addTween(this.actor, { height: 0,
-                                       time: 0.20,
-                                       transition: 'easeOutQuad',
-                                       onComplete: function() {
-                                           this.widget.hide();
-                                           this._in = false;
-
-                                           // clear all the search properties when hiding the entry
-                                           this._searchEntry.set_text('');
-                                       },
-                                       onCompleteScope: this });
+        this._in = false;
+        this.widget.set_reveal_child(false);
+        // clear all the search properties when hiding the entry
+        this._searchEntry.set_text('');
     }
 });
 
@@ -242,11 +232,11 @@ const Dropdown = new Lang.Class({
         this._matchView.connect('item-activated',
                                 Lang.bind(this, this._onItemActivated));
 
-        this.widget = new Gtk.Frame({ shadow_type: Gtk.ShadowType.IN });
-        this.actor = new GtkClutter.Actor({ contents: this.widget,
-                                            opacity: 0 });
-        let actorWidget = this.actor.get_widget();
-        actorWidget.get_style_context().add_class('documents-dropdown');
+        this.widget = new Gtk.Frame({ shadow_type: Gtk.ShadowType.IN,
+                                      halign: Gtk.Align.CENTER,
+                                      valign: Gtk.Align.START,
+                                      opacity: 0 });
+        this.widget.get_style_context().add_class('documents-dropdown');
 
         this._grid = new Gtk.Grid({ orientation: Gtk.Orientation.HORIZONTAL });
         this.widget.add(this._grid);
@@ -266,24 +256,19 @@ const Dropdown = new Lang.Class({
     show: function() {
         this.widget.show_all();
 
-        let parent = this.actor.get_parent();
-        parent.set_child_above_sibling(this.actor, null);
-
-        Tweener.addTween(this.actor, { opacity: 245,
-                                       time: 0.20,
-                                       transition: 'easeOutQuad' });
+        Tweener.addTween(this.widget, { opacity: 0.9,
+                                        time: 0.20,
+                                        transition: 'easeOutQuad' });
     },
 
     hide: function() {
-        this.widget.hide();
-        Tweener.addTween(this.actor, { opacity: 0,
-                                       time: 0.20,
-                                       transition: 'easeOutQuad',
-                                       onComplete: function() {
-                                           let parent = this.actor.get_parent();
-                                           parent.set_child_below_sibling(this.actor, null);
-                                       },
-                                       onCompleteScope: this });
+        Tweener.addTween(this.widget, { opacity: 0,
+                                        time: 0.20,
+                                        transition: 'easeOutQuad',
+                                        onComplete: function() {
+                                            this.widget.hide();
+                                        },
+                                        onCompleteScope: this });
     }
 });
 Signals.addSignalMethods(Dropdown.prototype);
@@ -309,6 +294,8 @@ const OverviewSearchbar = new Lang.Class({
         this._onActiveSourceChanged();
         this._onActiveTypeChanged();
         this._onActiveMatchChanged();
+
+        this._searchEntry.set_text(Application.searchController.getString());
     },
 
     createSearchWidgets: function() {
@@ -318,10 +305,9 @@ const OverviewSearchbar = new Lang.Class({
             Lang.bind(this, this._onTagClicked));
 
         // connect to search string changes in the controller
-        this._searchEntry.text = Application.searchController.getString();
         let searchChangedId = Application.searchController.connect('search-string-changed', Lang.bind(this,
             function(controller, string) {
-                this._searchEntry.text = string;
+                this._searchEntry.set_text(string);
             }));
 
         this._searchEntry.connect('destroy', Lang.bind(this,
